/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Original Author: Pete Collins <pete@mozdev.org>
Contributor(s):

***/

// STATUS: WORK IN PROGRESS

var JS_LIB_UNINSTALL_LOAD_MSG = 
                  "JS_LIB_UNISTALL library not loaded:\n"
                + " \tTo load use: chrome://jslib/content/jslib.js\n"
                + " \tThen: include('chrome://jslib/content/io/unsinstall.js');\n\n";

// Make sure jslib.js is loaded
if (typeof(JS_LIB_LOADED) == 'boolean')
{

// Ensure we are in a DOM window environment
if (typeof(window) == 'undefined') {
  JS_LIB_UNINSTALL_LOAD_MSG = "\n\nError: This library can only be used in a "
                              + "dom window environment\n\n";
  dump(JS_LIB_UNINSTALL_LOAD_MSG);
} else {
  // Load dependency lib files
  if (typeof(JS_FILE_LOADED)!='boolean')
    include(JS_LIB_PATH+'io/file.js');

  if (typeof(JS_DIRUTILS_LOADED)!='boolean')
    include(JS_LIB_PATH+'io/dirUtils.js');

  if (typeof(JS_DIR_LOADED)!='boolean')
    include(JS_LIB_PATH+'io/dir.js');

  if (typeof(JS_FILEUTILS_LOADED)!='boolean')
    include(JS_LIB_PATH+'io/fileUtils.js');

  // GLOBALS
  const JS_LIB_UNINSTALL_LOADED   = true;
  const JS_LIB_UNINSTALL_FILE     = 'uninstall.js';

  // CONSTRUCTOR
  function Uninstall(aPackageName) {

    if (!aPackageName) {
      jslibError(null, "Please enter a package name to uninstall",
               "NS_ERROR_XPC_NOT_ENOUGH_ARGS", JS_LIB_UNINSTALL_FILE);
      throw - Components.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    } else {
      this.mName = aPackageName;
      this.init();
    }
  }

  /*********** UNINSTALL ***************/
  Uninstall.prototype = {

    mName           : null,
    CHRM_REG_CID    : "@mozilla.org/chrome/chrome-registry;1",
    SKIN            : null,
    JAR             : null,

  init  : function () {
    this.SKIN   = this.mName+"/";
    this.JAR    = this.mName+".jar";
  },

  uninstallPackage : function () {
  if (this.removeRDFEntries() < 0 )
    return null;
  this.removePkg();
  return this.finishedDialog();
  },

  removePkg : function () {

    jslibDebug("uninstalling "+this.mName+"  . . . \n");
    var d   = new DirUtils();
    var f   = new File(d.getChromeDir());
    var fu  = new FileUtils();
    f.append(this.JAR);
    jslibDebug("jar path = "+ f.path +"\n");
    if (f.exists()) {
      f.remove();
    } else {
      f.initPath(fu.append(d.getChromeDir(), this.mName));
      jslibDebug("path = "+ f.path +"\n");
      if (!f.exists()) {
        jslibError(null, f.URL+" doesn't exist", 
                         "NS_ERROR_FILE_INVALID_PATH", 
                         JS_LIB_UNINSTALL_FILE);
        throw - C.results.NS_ERROR_FILE_INVALID_PATH;
      }
      if (f.exists() && !f.isSymlink()) {
        try { f.nsIFile.remove(true); } 
				catch (e) {}
			} else
        f.remove();
    }
    return 0;
  },

  removeRDFEntries : function () {
    var rv = Components.results.NS_OK;
    rv = this.removeOverlayEntry();
    // rv = this.removeAllPkgEntry();
    return rv;
  },

  clearMenu : function () {
    jslibDebug('clearMenu: '+this.mName+'\n');
    var menu = document.getElementById(this.mName);
    if (!menu) 
      return null;
    jslibDebug(menu);
    var rv;
    var sep  = document.getElementById(this.mName+"_separator");
    rv = sep.setAttribute("style", "display: none;");
    rv = menu.setAttribute("style", "display: none;");
    return rv;
  },

  finishedDialog : function () {
    const finish_msg  = this.mName+" is now uninstalled . . .\n";
    this.clearMenu();
    return alert(finish_msg);
  },

removeOverlayEntry : function () {
  const CONTAINER_NAME  = 'chrome://communicator/content/tasksOverlay.xul';
  const NODE_NAME       = 'chrome://'+this.mName+'/content/'+this.mName+'Overlay.xul';
  var leafs             = ['overlayinfo', 'communicator', 'content', 'overlays.rdf'];
  var f                 = new File((new DirUtils()).getChromeDir());
  //var f                 = new File('/tmp/overlays.rdf');
  for (var i = 0; i < leafs.length; i++)
    f.append(leafs[i]);
  return this.removeRDFContainer(f, CONTAINER_NAME, NODE_NAME);
},

removeAllPkgEntry : function () {
  const CONTAINER_NAME  = 'urn:mozilla:package:root';
  const NODE_NAME       = 'urn:mozilla:package:'+this.mName;
  var pkgFile           = 'all-packages.rdf';
  var rv = Components.results.NS_OK;
  //var f                 = new File((new DirUtils()).getChromeDir());
  //f.append(pkgFile);
  var f                 = new File('/tmp/all-packages.rdf');
  //for (var i = 0; i < leafs.length; i++)
    //f.append(leafs[i]);
  rv = this.removeRDFDescription(f, CONTAINER_NAME, NODE_NAME);
  rv = this.removeRDFContainer(f, CONTAINER_NAME, NODE_NAME);
  return rv;
},

removeRDFContainer : function (f, CONTAINER_NAME, NODE_NAME) {

  var C                       = Components;
  if (!(f && CONTAINER_NAME && NODE_NAME) )
    throw - C.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;

  const RDF_SRVC_PROG_ID      = '@mozilla.org/rdf/rdf-service;1';
  const RDF_CONTAINER_PROG_ID = '@mozilla.org/rdf/container;1';
  const RDF_DS_PROG_ID        = '@mozilla.org/rdf/datasource;1?name=xml-datasource';
  
  if (!f.exists()) {
    jslibError(null, f.URL+" doesn't exist", 
                      "NS_ERROR_FILE_INVALID_PATH", 
                      JS_LIB_UNINSTALL_FILE);
    throw - C.results.NS_ERROR_FILE_INVALID_PATH;
  }

  try {
    var rdf   = C.classes
                [RDF_SRVC_PROG_ID].getService(C.interfaces.nsIRDFService);
    var ds    = rdf.GetDataSource(f.URL);
    jslibDebug('ds = '+ds);
    var rem   = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
    jslibDebug('loaded = '+rem.loaded);
    jslibDebug('f.URL = '+f.URL);
    if (!rem.loaded) {
      rdf.UnregisterDataSource(ds);
      ds        = C.classes[RDF_DS_PROG_ID].
                  getService(C.interfaces.nsIRDFDataSource);
      rem       = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
      rem.Init(f.URL);
    }
    jslibDebug(rem);
    rem.Refresh(true)
    var rs  = rdf.GetResource(CONTAINER_NAME);
    var container = C.classes[RDF_CONTAINER_PROG_ID].
                         getService(C.interfaces.nsIRDFContainer);
    jslibDebug("before Init Container");
    container.Init(rem, rs);
    jslibDebug(container.GetCount());
/***********
    els = container.GetElements();
    while (els.hasMoreElements())
      jslibDebug(els.getNext());
************/
    jslibDebug(NODE_NAME);
    var el = rdf.GetLiteral(NODE_NAME); // remove if it's a literal
    container.RemoveElement(el, true);
    el = rdf.GetResource(NODE_NAME); // remove it it's a resource
    container.RemoveElement(el, true);
    jslibDebug(container.GetCount());
    rem.Flush()
    rem.Refresh(true)
  } catch (e) {
      jslibError(e, "unexpected error", 
                      "NS_ERROR_UNEXPECTED", 
                      JS_LIB_UNINSTALL_FILE);
      return - C.results.NS_ERROR_UNEXPECTED;
  }

  return 0;
},

removeRDFDescription : function (f, CONTAINER_NAME, NODE_NAME) {

  var C                       = Components;
/*******
  if (!(f && CONTAINER_NAME && NODE_NAME) )
    throw - C.results.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
*******/

  const RDF_SRVC_PROG_ID      = '@mozilla.org/rdf/rdf-service;1';
  const RDF_CONTAINER_PROG_ID = '@mozilla.org/rdf/container;1';
  const RDF_DS_PROG_ID        = '@mozilla.org/rdf/datasource;1?name=xml-datasource';
  
  if (!f.exists()) {
    jslibError(null, f.URL+" doesn't exist", 
                      "NS_ERROR_FILE_INVALID_PATH", 
                      JS_LIB_UNINSTALL_FILE);
    throw - C.results.NS_ERROR_FILE_INVALID_PATH;
  }

  try { 
    var rdf   = C.classes[RDF_SRVC_PROG_ID].
                getService(C.interfaces.nsIRDFService);
    var ds    = rdf.GetDataSource(f.URL);
    jslibDebug('ds = '+ds);
    var rem   = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
    jslibDebug('loaded = '+rem.loaded);
    if (!rem.loaded) {
      rdf.UnregisterDataSource(ds);
      ds        = C.classes[RDF_DS_PROG_ID].
                  getService(C.interfaces.nsIRDFDataSource);
      rem       = ds.QueryInterface(C.interfaces.nsIRDFRemoteDataSource);
      rem.Init(f.URL);
    }
    jslibDebug('f.URL = '+f.URL);

    var src   = rdf.GetResource(CONTAINER_NAME); 
    var node  = rdf.GetResource(NODE_NAME); 
    var prop  = rdf.GetResource(NODE_NAME); 

    rem.Refresh(true)

    var a   = rdf.GetResource('c:baseURL');
    var b   = rdf.GetLiteral('resource:/chrome/theme_builder/content/'); 
    jslibDebug("Checking for Assertion "+node.Value+"\n"+a.Value+"\n"+b.Value);
    var assertion = rem.HasAssertion(node, a, b, true);
    jslibDebug('HasAssertion = '+assertion);
    rem.Unassert(node, a, b, true);
    rem.Flush();
    rem.Refresh(true);
    return 0;

/**************************
    while (resources.hasMoreElements())
      jslibDebug(resources.getNext().Value);
    //var node  = rdf.GetResource('urn:mozilla:package:fubar');
    var resA    = rdf.GetResource('c:baseFoo');
    var node1   = rdf.GetLiteral('pete'); 
    var res   = rdf.GetResource('http://www.mozilla.org/rdf/chrome#');
    //var ass   =  rem.HasAssertion(res, res, node, true);
jslibDebug('HasAssertion = '+rem.HasAssertion(node, resA, node1, true));
    //var ass   =  rem.Unassert(node, resA, node1, true);
    var ass   =  rem.Assert(node, resA, node1, true);
    ass   =  rem.Assert(node, resB, node2, true);
    rem.Flush();
// return 0;
    ass   =  rem.HasAssertion(node, resA, node1, true);
    jslibDebug("HasAssertion = "+ass);
    var node3 = rdf.GetLiteral('resource:/chrome/theme_builder/content/');
jslibDebug("Unassert "+node.Value+" "+resB.Value+" "+node2.Value);
    ass   =  rem.Unassert(node, resB, node1, true);
    rem.Flush();
*********************************/
  } catch (e) {
    jslibError(e, "error failure", 
                      "NS_ERROR_FAILURE", 
                      JS_LIB_UNINSTALL_FILE);
    throw - C.results.NS_ERROR_FAILURE;
  }
  return 0;
}

  /*********** UNINSTALL ***************/
  } // END CLASS
  jslibDebug('*** load: '+JS_LIB_UNINSTALL_FILE+' OK');

} // END BLOCK DOM WINDOW CHECK  

// END BLOCK JS_LIB_LOADED CHECK 
// If jslib base library is not loaded, dump this error.
} else {
  dump(JS_LIB_UNINSTALL_LOAD_MSG);
}
